@@ -1,7 +1,7 @@
 import sys
 from functools import partial
 from xdoctest.dynamic_analysis import get_parent_frame, get_stack_frame
-from . import util
+from xdev import util
 import time
 
 
@@ -13,6 +13,8 @@ def embed(parent_locals=None, parent_globals=None, exec_lines=None,
 
     """
     import IPython
+    import xdev  # NOQA
+    import xdev as xd  # NOQA
 
     if parent_globals is None:
         parent_globals = get_parent_frame(n=n).f_globals
