@@ -283,8 +283,12 @@ def difftext(text1, text2, context_lines=0, ignore_whitespace=False,
     return text
 
 
-def tree(cwd=None, max_files=0):
+def tree_repr(cwd=None, max_files=100, dirblocklist=None, show_nfiles='auto',
+              return_text=False, return_tree=False, pathstyle='name',
+              with_type=False, colors=not ub.NO_COLOR):
     """
+    Filesystem tree representation
+
     Like the unix util tree, but allow writing numbers of files per directory
     when given -d option
 
@@ -293,74 +297,143 @@ def tree(cwd=None, max_files=0):
     Args:
         cwd : directory to print
         max_files : maximum files to print before supressing a directory
+        pathstyle (str): can be rel, name, or abs
+        return_tree (bool): if True return the tree
+        return_text (bool): if True return the text
+        colors (bool): if True use rich
+
+    SeeAlso:
+        xdev.tree - generator
+
+    Ignore:
+        >>> import xdev
+        >>> xdev.tree_repr()
     """
     import os
-    from os.path import join, relpath
+    from os.path import join, relpath, basename
     if cwd is None:
         cwd = os.getcwd()
 
     import networkx as nx
     tree = nx.OrderedDiGraph()
 
+    from xdev.patterns import MultiPattern
+    if dirblocklist is not None:
+        dirblocklist = MultiPattern.coerce(dirblocklist, hint='glob')
+
+    def _make_label(p):
+        if pathstyle == 'rel':
+            pathrep = relpath(p, cwd)
+        elif pathstyle == 'name':
+            pathrep = basename(p)
+        elif pathstyle == 'abs':
+            pathrep = p
+        else:
+            KeyError(pathstyle)
+
+        types = []
+        islink = os.path.islink(p)
+        isdir = os.path.isdir(p)
+        isfile = os.path.isfile(p)
+        isbroken = False
+        scolor = ''
+        tcolor = ''
+        L_scolor = ''
+        L_tcolor = ''
+        if islink:
+            if colors:
+                L_scolor = '[cyan]'
+                L_tcolor = '[/cyan]'
+            types.append('L')
+            if not isfile and not isdir:
+                isbroken = True
+                if isbroken:
+                    if colors:
+                        L_scolor = '[red]'
+                        L_tcolor = '[/red]'
+                types.append('B')
+
+        if isfile:
+            if colors:
+                scolor = '[reset]'
+                tcolor = '[/reset]'
+            types.append('F')
+        if isdir:
+            if colors:
+                scolor = '[blue]'
+                tcolor = '[/blue]'
+            types.append('D')
+
+        if islink:
+            target = os.readlink(p)
+            pathrep = L_scolor + pathrep + L_tcolor + ' -> ' + scolor + target + tcolor
+        else:
+            pathrep = scolor + pathrep + tcolor
+
+        if with_type:
+            typelabel = ''.join(types)
+            return f'({typelabel}) ' + pathrep
+        else:
+            return pathrep
+
+    # TODO: rectify with "find"
     for root, dnames, fnames in os.walk(cwd):
+
+        if dirblocklist is not None:
+            dnames[:] = [
+                dname for dname in dnames if not dirblocklist.match(dname)]
+
         dnames[:] = sorted(dnames)
         tree.add_node(root)
 
+        too_many_files = max_files is not None and len(fnames) >= max_files
+
+        if show_nfiles == 'auto':
+            show_nfiles_ = too_many_files
+        else:
+            show_nfiles_ = show_nfiles
+
         num_files = len(fnames)
-        prefix = '[ {} ]'.format(num_files)
+        if show_nfiles_:
+            prefix = '[ {} ] '.format(num_files)
+        else:
+            prefix = ''
 
-        pathrep = relpath(root, cwd)
-
-        label = '{} {}'.format(prefix, pathrep)
+        label = '{}{}'.format(prefix, _make_label(root))
 
         tree.nodes[root]['label'] = label
 
-        if len(fnames) < max_files:
+        if not too_many_files:
             for fname in fnames:
                 fpath = join(root, fname)
+                tree.add_node(fpath)
+                tree.nodes[fpath]['label'] = _make_label(fpath)
                 tree.add_edge(root, fpath)
 
         for dname in dnames:
             dpath = join(root, dname)
+            tree.add_node(dpath)
+            tree.nodes[dpath]['label'] = _make_label(dpath)
             tree.add_edge(root, dpath)
 
-    _print_forest(tree)
+    from xdev.util_networkx import write_network_text
+    import io
+    file = io.StringIO()
+    write_network_text(tree, file)
+    file.seek(0)
+    text = file.read()
 
+    info = {}
 
-def _print_forest(graph):
-    """
-    Nice ascii representation of a forest
-
-    Ignore:
-        graph = nx.balanced_tree(r=2, h=3, create_using=nx.DiGraph)
-        _print_forest(graph)
-
-        graph = CategoryTree.demo('coco').graph
-        _print_forest(graph)
-    """
-    if len(graph.nodes) == 0:
-        print('--')
-        return
-
-    import networkx as nx
-    assert nx.is_forest(graph)
-
-    def _recurse(node, indent='', islast=False):
-        if islast:
-            this_prefix = indent + '└── '
-            next_prefix = indent + '    '
+    if return_text:
+        info['text'] = text
+    else:
+        if colors:
+            from rich import print as rprint
+            rprint(text)
         else:
-            this_prefix = indent + '├── '
-            next_prefix = indent + '│   '
-        label = graph.nodes[node].get('label', node)
-        print(this_prefix + str(label))
-        graph.succ[node]
-        children = graph.succ[node]
-        for idx, child in enumerate(children, start=1):
-            islast_next = (idx == len(children))
-            _recurse(child, indent=next_prefix, islast=islast_next)
+            print(text)
 
-    sources = [n for n in graph.nodes if graph.in_degree[n] == 0]
-    for idx, node in enumerate(sources, start=1):
-        islast_next = (idx == len(sources))
-        _recurse(node, indent='', islast=islast_next)
+    if return_tree:
+        info['tree'] = tree
+    return info
